/*
  Power Remote Display V1
  ***********************

  Affiche la consommation  ou la surproduction de la maison en Wh
  
  Détails sur :
  https://f1atb.fr  in the Home Automation / Domotique section

  F1ATB Janvier 2023

  GNU General Public License v3.0

*/

//Libraries
//TFT Display
#include <TFT_eSPI.h>  // Graphics and font library for ST7735 driver chip
#include <SPI.h>
#include "Free_Fonts.h"

//WIFI
#include <WiFi.h>
#include <WiFiClient.h>
#include <NTPClient.h>
#include <WiFiUdp.h>


/*
  Données à adapter à votre configuration de réseau local
  -------------------------------------------------------
*/

//WIFI
const char* ssid = "WIFI";          //Set here your WIFI SSID
const char* password = "123456789";  //Set here the WIFI password

// Set your Static IP address of the ESP32
IPAddress local_IP(192, 168, 0, 210);
// Set your Gateway IP address
IPAddress gateway(192, 168, 0, 254);
IPAddress subnet(255, 255, 255, 0);
IPAddress primaryDNS(8, 8, 8, 8);    //optional
IPAddress secondaryDNS(8, 8, 4, 4);  //optional

// Client of Power Router. Adresse source des données de puissance Routeur Solaire V1 ou V2 F1ATB 2022
const char* hostSource = "192.168.0.208";
const int httpPort = 80;
/*
  Fin zone paramètre à adapter
  ----------------------------
*/

//PINS - GPIO
const int GPIO0 = 0;     //Bouton
const int GPIO35 = 35;   //Bouton
const int GPIO_IR = 33;  //Détecteur présence


//TFT
TFT_eSPI tft = TFT_eSPI();  // Invoke library, pins defined in User_Setup.h

//VARIABLES


float tabData[11];
int tabVolt[101];
int tabAmp[101];
int But0 = 1;
int But35 = 1;
bool AfficheurOn = true;
bool GoodIRpulse =true;
String Heure;

//Variable sauvegardée pendant le Deep Sleep
RTC_DATA_ATTR int LastW = 0;


//Internal Timers
unsigned long previousDataMillis;
unsigned long previousWifiMillis;
unsigned long previousIRMillis;



WiFiUDP ntpUDP;
NTPClient temps(ntpUDP, "fr.pool.ntp.org", 3600, 300000);  // External timeb  (Udp,pool,offset in second 3600=winter in europe,update in ms)


// Power Router client
//********************
//Recherche de la puissance auprès du routeur solaire de F1ATB
//Vous pouvez adapter cette partie à tout type de source de données accessible par une url
void Lecture_Puissance() {
  String url;
  // Use WiFiClient class to create TCP connections
  WiFiClient client;
  if (!client.connect(hostSource, httpPort)) {
    Serial.println("Connexion vers " + String(hostSource) + " échoué");
    delay(100);
    return;
  }
  // We now create a URI for the request
  url = "/ajax_data";

  client.print(String("GET ") + url + " HTTP/1.1\r\n" + "Host: " + hostSource + "\r\n" + "Connection: close\r\n\r\n");
  unsigned long timeout = millis();
  while (client.available() == 0) {
    if (millis() - timeout > 5000) {
      Serial.println(">>> Client Timeout HostSource : " + String(hostSource));
      tft.println(">>> Client Timeout ! HostSource : " + String(hostSource));
      client.stop();
      return;
    }
  }

  // Read all the lines of the reply from server
  while (client.available()) {
    String line = client.readStringUntil('\r');
    String MainData[3];
    int StringCount = 0;
    if (line.indexOf('|') > 0) {  // split line '|' 3 Champs de caractères séparés par "|"
      while (line.length() > 0) {
        int index = line.indexOf('|');
        if (index == -1) {  // Not found
          MainData[StringCount++] = line;
          break;
        } else {
          MainData[StringCount++] = line.substring(0, index);
          line = line.substring(index + 1);
        }
      }

      line = MainData[0];  //Champ des différentes mesures
      StringCount = 0;
      while (line.length() > 0) {
        int index = line.indexOf(',');  // split line ','
        if (index == -1) {              // Not found
          tabData[StringCount++] = line.toFloat();
          break;
        } else {
          tabData[StringCount++] = line.substring(0, index).toFloat();
          line = line.substring(index + 1);
        }
      }

      line = MainData[1];  //Champ des 100 mesures de tension + ',0'
      StringCount = 0;
      while (line.length() > 0) {
        int index = line.indexOf(',');
        if (index == -1) {  // Not found
          tabVolt[StringCount++] = line.toInt();
          ;
          break;
        } else {
          tabVolt[StringCount++] = line.substring(0, index).toInt();
          line = line.substring(index + 1);
        }
      }

      line = MainData[2];  //Champ des 100 mesures de courant + ',0'
      StringCount = 0;
      while (line.length() > 0) {
        int index = line.indexOf(',');
        if (index == -1) {  // Not found
          tabAmp[StringCount++] = line.toInt();
          ;
          break;
        } else {
          tabAmp[StringCount++] = line.substring(0, index).toInt();
          line = line.substring(index + 1);
        }
      }
    }
  }
  LastW = int(tabData[2]);  //Watt value
}



void Read_Hour() {
  //Time Update / de l'heure
  temps.update();
  Heure = temps.getFormattedTime();
}


void LireIR() {  //Capteur Infra Rouge
  int IR = digitalRead(GPIO_IR);
  if (IR == 1) {  //Présence source de chaleur en mouvement
    previousIRMillis = millis();
    if (previousIRMillis >40000) GoodIRpulse=true;
    if ( GoodIRpulse) AfficheurOn=true;
  } else {
     GoodIRpulse =true;
  }
  if (millis() - previousIRMillis > 10000) {  //Attente On 10 secondes                                        //Go to sleep now
    Serial.println("Going to sleep now");     // Mise en veille profonde
    tft.fillScreen(TFT_BLACK);
    esp_deep_sleep_start();
  }
}

//Sorties Afficheur
void Affichage() {
  if (But0 == 0) {  //Courbes
    TraceCourbe();
  } else {
    if (But35 == 0) {  //Tableau de Valeurs
      Trace_Valeurs();
    } else {  //Page principale
      TraceWatt();
    }
  }
}
void TraceWatt() {
  tft.setFreeFont(FSB24);
  tft.fillScreen(TFT_NAVY);
  tft.drawCentreString(String(LastW) + " W", 120, 40, GFXFF);
  tft.setFreeFont(FS9);
  tft.drawString(Heure, 8, 3, 1);
  if (tabData[2] > 0) {
    int WX = floor(tabData[2] * 240 / 12000);  //Max conso 12kW
    tft.fillRect(0, 110, WX, 20, TFT_ORANGE);
  } else {
    int WX = floor(tabData[2] * 240 / 5000);  // Pas la meme échelle pour production Max 5kW
    tft.fillRect(240 + WX, 110, -WX, 20, TFT_GREEN);
  }
}
void Trace_Valeurs() {
  tft.setFreeFont(FS9);
  tft.fillScreen(TFT_NAVY);
  tft.drawString("Ueff : " + String(tabData[0]) + " V", 8, 10, 1);
  tft.drawString("Ieff  : " + String(tabData[1]) + " A", 8, 30, 1);
  tft.drawString("PW : " + String(int(tabData[2])) + " W", 8, 50, 1);
  tft.drawString("PVA : " + String(int(tabData[3])) + " VA", 8, 70, 1);
  tft.drawString("Cos(phi) : " + String(tabData[4]), 8, 90, 1);
  tft.drawString("kWh : " + String(tabData[5]), 8, 110, 1);
}
void TraceCourbe() {
  int Vmin = 30000;
  int Vmax = -30000;
  int Imin = 30000;
  int Imax = -30000;
  for (int i = 0; i < 100; i++) {
    Vmin = min(Vmin, tabVolt[i]);
    Vmax = max(Vmax, tabVolt[i]);
    Imin = min(Imin, tabAmp[i]);
    Imax = max(Imax, tabAmp[i]);
  }
  Vmax = 1.05 * max(Vmax, -Vmin);
  Vmin = -Vmax;
  Imax = 1.05 * max(Imax, -Imin);
  Imin = -Imax;
  tft.fillScreen(TFT_NAVY);
  int X0 = 0;
  int Y0 = map(tabVolt[0], Vmin, Vmax, 135, 0);
  for (int i = 1; i < 100; i++) {
    int X1 = i * 240 / 100;
    int Y1 = map(tabVolt[i], Vmin, Vmax, 135, 0);
    tft.drawLine(X0, Y0, X1, Y1, TFT_RED);
    X0 = X1;
    Y0 = Y1;
  }
  X0 = 0;
  Y0 = map(tabAmp[0], Imin, Imax, 135, 0);
  for (int i = 1; i < 100; i++) {
    int X1 = i * 240 / 100;
    int Y1 = map(tabAmp[i], Imin, Imax, 135, 0);
    tft.drawLine(X0, Y0, X1, Y1, TFT_GREEN);
    X0 = X1;
    Y0 = Y1;
  }
  tft.setFreeFont(FS9);
  tft.setTextColor(TFT_RED, TFT_NAVY);
  tft.drawString("Ueff : " + String(tabData[0]) + " V", 8, 70, 1);
  tft.setTextColor(TFT_GREEN, TFT_NAVY);
  tft.drawString("Ieff  : " + String(tabData[1]) + " A", 8, 90, 1);
  tft.setTextColor(TFT_WHITE, TFT_NAVY);
  tft.drawString("PVA : " + String(int(tabData[3])) + " VA", 8, 110, 1);
}


// SETUP
//*******
void setup() {
  esp_sleep_wakeup_cause_t source_reveil;
  source_reveil = esp_sleep_get_wakeup_cause();  //0= boot, 2=interruption capteur IR, 4 = interruption timer. Valeurs différentes de la doc Espressif ?
  AfficheurOn=true;
  if (source_reveil == 4) {
    AfficheurOn = false;
  }
  Serial.begin(115200);
  Serial.println("Booting");
  tft.init();
  tft.setRotation(1);
  tft.setTextColor(TFT_WHITE, TFT_NAVY);
  Serial.println("Booting");
  Serial.println("Wake up : " + String(source_reveil));
  int IR = digitalRead(GPIO_IR);
   Serial.println("IR at wake up : " + String(IR));
  if (AfficheurOn) TraceWatt();
  // Configures static IP address
  if (!WiFi.config(local_IP, gateway, subnet, primaryDNS, secondaryDNS)) {
    Serial.println("STA Failed to configure");
    tft.println("STA Failed to configure");
  }
  //WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while (WiFi.waitForConnectResult() != WL_CONNECTED) {

    tft.println("Wifi Connection Failed! Rebooting...");
    delay(5000);
    ESP.restart();
  }

  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.println("HostSource : " + String(hostSource));



  //Various Timers

  previousDataMillis = 0;
  previousWifiMillis = millis();
  previousIRMillis = millis();



  pinMode(GPIO0, INPUT);
  pinMode(GPIO35, INPUT);
  pinMode(GPIO_IR, INPUT);


  //External timer to obtain the Hour
  temps.begin();

  //2 modes de Deep Sleep
  esp_sleep_enable_ext0_wakeup(GPIO_NUM_33, 1);  //Réveil si presence Infra Rouge passe à 1
  esp_sleep_enable_timer_wakeup(600000000);       //Réveil périodique en uS sans affichage pour avoir immediatement un résultat à afficher lors d'un reveil IR
  Read_Hour();
   IR = digitalRead(GPIO_IR);
   Serial.println("IR at end setup : " + String(IR));
   GoodIRpulse =true;
   if (source_reveil==4 && IR==1) GoodIRpulse =false; //IR déclenché par le wifi. Perturbe la lecture de la puissance en veille écran eteint
}

// LOOP
//******
void loop() {

  if (digitalRead(GPIO0) != But0 || digitalRead(GPIO35) != But35) {  //Nouvelle page demandée
    tft.fillScreen(TFT_DARKGREY);
    But0 = digitalRead(GPIO0);
    But35 = digitalRead(GPIO35);
    Affichage();
    previousDataMillis = millis() - 2000;  // Accélerer l'appel au serveur de données
  }

  LireIR();

  if ((millis() - previousDataMillis >= 2000)) {
    int IR = digitalRead(GPIO_IR);
    
   Serial.println("IR loop : " + String(IR));
    Read_Hour();
    Lecture_Puissance();  //Call Power Router for last data
    Serial.println(String(LastW) + " W");
    //Rafraichissement Affichage
    if (AfficheurOn && GoodIRpulse) Affichage();
    previousDataMillis = millis();
  }

  delay(10);
}
